function Round(num, numDecimalPlaces)
    local mult = 10^(numDecimalPlaces or 0)
    local number = math.floor(num * mult + 0.5) / mult
    if (number % 1) == 0 and numDecimalPlaces ~= 0 then
        return tostring(number .. ".0")
    else
        return number
    end
end

function tablelength(T)
    local count = 0
    if T ~= nil then
        for _ in pairs(T) do count = count + 1 end
    end
    return count
end

function Tablelength(T)
    return tablelength(T)
end

function Compare(a, b)
    if a ~= nil and b ~= nil then
        return a[2] < b[2]
    end
end

math.round = function(number) return math.floor(number + 0.5) end

function ChatOutput(s) Turbine.Shell.WriteLine(string.format("%s%s", "<rgb=#FF9900><Prime></rgb> ", tostring(s))) end

function DirectChatOutput(s) Turbine.Shell.WriteLine(s) end

function Print(s) Turbine.Shell.WriteLine(tostring(s)) end

function GetRandomColour() return(Turbine.UI.Color(math.random(), math.random(), math.random())) end

function FormatBigNumber(number)
    if number == nil then return 0 end
    if number < 1000 then
        return string.format("%d", number)
    elseif number >= 1000 and number < 1000000 then
        return string.format("%.1fK", math.floor(number / 100) / 10)
    elseif number >= 1000000 and number < 1000000000 then
        return string.format("%.1fM", math.floor(number / 100000) / 10)
    elseif number >= 1000000000 and number < 1000000000000 then 
        return string.format("%.1fB", math.floor(number / 100000000) / 10)
    elseif number >= 1000000000000 then
        return string.format("%.1fT", math.floor(number / 100000000000) / 10)
    else
        return number
    end
end

function FormatBigNumberVitals(number)
    if number == nil then return 0 end

    local suffixes = {"", "K", "M", "B", "T"}
    local index = 1

    while number >= 1000 and index < #suffixes do
        number = number / 1000
        index = index + 1
    end

    return string.format("%.1f%s", number, suffixes[index])
end


function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function TimeDeltaToString(time)
    local minutes = math.floor(time / 60)
    local seconds = math.floor(time % 60)
    local milliseconds = math.floor((time * 1000) % 1000)

    return string.format("%02d:%02d.%03d", minutes, seconds, milliseconds)
end


function CombatTimeDeltaToString(time)
    return string.format("%01d:%02d", math.floor(time / 60), math.floor(time % 60))
end

function FormatTimeString(time)
    local minutes = math.floor(time / 60)
    local seconds = string.format("%02.f", math.floor(time % 60))
    local milliseconds = string.format("%02.f", math.floor(time * 100 % 100))

    return(minutes .. ":" .. seconds .. "." .. milliseconds)
end

function FormatTimeStringNoMilli(time)
    local minutes = math.floor(time / 60)
    local seconds = string.format("%02.f", math.floor(time % 60))

    return(minutes .. ":" .. seconds)
end

function FormatTimeStringWritten(time)
    local hours = math.floor(time / 3600)
    local minutes = math.floor((time % 3600) / 60)
    local seconds = math.floor(time % 60)

    local output = ""
    if hours > 0 then
        output = output .. math.floor(hours) .. " hours "
    end

    if minutes > 0 then
        output = output .. math.floor(minutes) .. " minutes "
    end

    output = output .. seconds .. " seconds"
    

    return output
end


function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback)
        else
            object[event] = {object[event], callback}
        end
    end
    return callback
end

function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil
    else
        if (type(object[event]) == "table") then
            for i = 1, #object[event] do
                if (object[event][i] == callback) then
                    table.remove(object[event], i)
                    break
                end
            end
        end
    end
end

function math.clamp(value, lower, upper)
    return math.max(lower, math.min(upper, value))
end

function BoolToString(bool) if bool then return ("True") else return("False") end end

function IsNumber(x)
    if tonumber(x) then return true end
    return false
end

function StringifyDuration(duration, round)
    if not duration then return "" end
    if round then
        if duration <= 60 then
            return string.format("%." .. round .. "f", duration)
        elseif duration <= 3600 then
            return string.format("%i:%02." .. round .. "f", duration / 60, duration % 60, duration % 60)
        else
            return string.format("%i:%02d:%02." .. round .. "f", duration / 3600, (duration % 3600) / 60, duration % 60)
        end
    else
        if duration <= 60 then
            return string.format("%i", duration)
        elseif duration <= 3600 then
            return string.format("%i:%02d", duration / 60, duration % 60)
        else
            return string.format("%i:%02d:%02d", duration / 3600, (duration % 3600) / 60, duration % 60)
        end
    end
end


function PrintTable(T)
    if T == nil or type(T) ~= "table" then return end

    for key, value in pairs(T) do
        if type(key) == "table" then 
            PrintTable(key) 
        else
            ChatOutput("Key: " .. tostring(key))
        end

        if type(value) == "table" then 
            PrintTable(value) 
        else
            ChatOutput("Value: " .. tostring(value))
        end
    end
end

function CalculateTotalTimePlayed(timeString)
   
    local seconds = 0
    local minutes = 0
    local hours = 0
    local days = 0
    local weeks = 0
    local months = 0
    local years = 0
    
    for value, unit in timeString:gmatch("(%d+)%s?(%a+)") do
        local numericValue = tonumber(value)
        
        if unit == "second" or unit == "seconds" then
            seconds = numericValue
        elseif unit == "minute" or unit == "minutes" then
            minutes = numericValue
        elseif unit == "hour" or unit == "hours" then
            hours = numericValue
        elseif unit == "day" or unit == "days" then
            days = numericValue
        elseif unit == "week" or unit == "weeks" then
            weeks = numericValue
        elseif unit == "month" or unit == "months" then
            months = numericValue
        elseif unit == "year" or unit == "years" then
            years = numericValue
        end
    end
    
    return  seconds + (minutes * 60) + (hours * 3600) + (days * 86400) + (weeks * 604800) + (months * 2628000) + (years * 31536000)

end
